@@ -1,5 +1,5 @@
 from abc import ABC
-from typing import Union
+from typing import Optional, Union
 
 import numpy
 
@@ -192,12 +192,7 @@ def _check_expr_incompatible_types(a, b):
         if isinstance(v, Value):
             return type(v.value)
         if isinstance(v, ColumnReference):
-            if (v.view is None) or (v.view.column_types is None) or (len(v.view.column_types) <= 0):
-                return None
-            try:
-                return v.view.column_types[v.column_name]
-            except KeyError:
-                return None
+            return None
         if isinstance(v, PreTerm):
             return None
         return None  # dunno
@@ -600,6 +595,17 @@ class Term(PreTerm, ABC):
     def co_equalizer(self, x):
         return self.__op_expr__("co_equalizer", x, inline=False, method=True)
 
+    def mapv(self, value_map, default_value):
+        """
+        Map values.
+        """
+
+        assert isinstance(value_map, DictTerm)
+        assert isinstance(default_value, Value)
+        return self.__triop_expr__(
+            "mapv", x=value_map, y=default_value, inline=False, method=True
+        )
+
     # fns that had been in bigquery_user_fns
 
     def as_int64(self):
@@ -788,6 +794,44 @@ class ListTerm(PreTerm):
             ti.get_column_names(columns_seen)
 
 
+class DictTerm(PreTerm):
+    # derived from PreTerm as this is not combinable
+    # only holds values
+    def __init__(self, value):
+        assert isinstance(value, dict)
+        self.value = value.copy()
+        PreTerm.__init__(self)
+
+    def is_equal(self, other):
+        # can't use == as that builds a larger expression
+        if not isinstance(other, DictTerm):
+            return False
+        return self.value == other.value
+
+    def get_views(self):
+        views = list()
+        return views
+
+    def replace_view(self, view):
+        return DictTerm(self.value)
+
+    def evaluate(self, data_frame):
+        return self.value.copy()
+
+    def to_python(self, *, want_inline_parens: bool = False) -> PythonText:
+        def li_to_python(value):
+            try:
+                return str(value.to_python(want_inline_parens=False))
+            except AttributeError:
+                return value.__repr__()
+
+        terms = [li_to_python(k) + ': ' + li_to_python(v) for k, v in self.value.items()]
+        return PythonText("{" + ", ".join(terms) + "}", is_in_parens=False)
+
+    def get_column_names(self, columns_seen):
+        pass
+
+
 def enc_value(value):
     if isinstance(value, PreTerm):
         return value
@@ -801,20 +845,21 @@ def enc_value(value):
 class ColumnReference(Term):
     """class to represent referring to a column"""
 
-    view: any
+    view: Optional["data_algebra.data_ops.ViewRepresentation"]
     column_name: str
 
     def __init__(self, view, column_name):
         self.view = view
         self.column_name = column_name
         assert isinstance(column_name, str)
-        if view is not None:
-            if column_name not in view.column_set:
-                raise KeyError(
-                    "column_name '"
-                    + str(column_name)
-                    + "' must be a column of the given view"
-                )
+        # # an invariant we want, but don't want the expense of checking it against lists
+        # if view is not None:
+        #     if column_name not in view.column_names:
+        #         raise KeyError(
+        #             "column_name '"
+        #             + str(column_name)
+        #             + "' must be a column of the given view"
+        #         )
         Term.__init__(self)
 
     def evaluate(self, data_frame):
