@@ -97,7 +97,7 @@ def _type_safe_is_in(a, b):
     if len(b) > 0:
         type_a = data_algebra.util.guess_carried_scalar_type(a)
         type_b = {data_algebra.util.map_type_to_canonical(type(v)) for v in b}
-        if len(type_b) > 1:
+        if not data_algebra.util.compatible_types(type_b):
             raise TypeError(f"multiple types in set: {type_b}")
         type_b = list(type_b)[0]
         if not data_algebra.util.compatible_types([type_a, type_b]):
@@ -105,6 +105,29 @@ def _type_safe_is_in(a, b):
     return numpy.isin(a, b)
 
 
+def _map_v(a, value_map, default_value):
+    if len(value_map) > 0:
+        type_a = data_algebra.util.guess_carried_scalar_type(a)
+        type_k = {data_algebra.util.map_type_to_canonical(type(v)) for v in value_map.keys()}
+        if not data_algebra.util.compatible_types(type_k):
+            raise TypeError(f"multiple types in dictionary keys: {type_k} in mapv()")
+        type_k = list(type_k)[0]
+        if not data_algebra.util.compatible_types([type_a, type_k]):
+            raise TypeError(f"can't map {type_a} from a dict of {type_k}'s in mapv()")
+        type_v = {data_algebra.util.map_type_to_canonical(type(v)) for v in value_map.values()}
+        if not data_algebra.util.compatible_types(type_v):
+            raise TypeError(f"multiple types in dictionary values: {type_v} in mapv()")
+        type_v = list(type_v)[0]
+        type_d = data_algebra.util.guess_carried_scalar_type(default_value)
+        if not data_algebra.util.compatible_types([type_d, type_v]):
+            raise TypeError(f"default is {type_d} for {type_v} values in mapv()'s")
+    # https://pandas.pydata.org/docs/reference/api/pandas.Series.map.html
+    a = data_algebra.default_data_model.pd.Series(a)
+    a = a.map(value_map, na_action='ignore')
+    a[data_algebra.default_data_model.bad_column_positions(a)] = default_value
+    return numpy.array(a.values)
+
+
 def _k_and(*args):
     res = args[0]
     for i in range(1, len(args)):
@@ -178,6 +201,7 @@ def populate_impl_map(data_model):
         "co_equalizer": lambda a, b: data_algebra.connected_components.connected_components(
             a, b
         ),
+        "mapv": _map_v,
         # fns that had been in bigquery_user_fns
         # x is a pandas Series
         "as_int64": lambda x: x.astype("int64").copy(),
@@ -304,14 +328,6 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         # make an index-free copy of the data to isolate side-effects and not deal with indices
         res = df.loc[:, columns_using]
         res = res.reset_index(drop=True)
-        # check type compatibility
-        if((op.column_types is not None) and (len(op.column_types) > 0)
-                and (res.shape[0] > 0) and (res.shape[1] > 0)):
-            types_seen = data_algebra.util.guess_column_types(res)
-            for c in columns_using:
-                assert data_algebra.util.compatible_types(
-                    {types_seen[c]}.union({op.column_types[c]})
-                )
         return res
 
     def columns_to_frame_(self, cols, *, target_rows=0):
